#include "combined_grade_list.hpp"

#include "user_grade_config.hpp"

#include <algorithm>
#include <cassert>
#include <functional>
#include <ranges>

void CombinedGradeList::recalculate() {

    const auto &closedMaxRef =
        std::get<const ChangedMaxPoints<GradingType::Closed> &>(
            this->mParentRefs);
    const auto &openMaxRef =
        std::get<const ChangedMaxPoints<GradingType::Open> &>(
            this->mParentRefs);

    const auto &closedGradesRef =
        std::get<const GradeList<GradingType::Closed> &>(this->mParentRefs);
    const auto &openGradesRef =
        std::get<const GradeList<GradingType::Open> &>(this->mParentRefs);

    const auto &configRef =
        std::get<const UserGradeConfig &>(this->mParentRefs);

    // Since we want to compute the average for each pair of grades, the
    // two lists have to have the same size.
    assert(closedGradesRef.mGrades.size() == openGradesRef.mGrades.size());

    const auto combinedMax = closedMaxRef.mChangedMaxPointBoundary +
                             openMaxRef.mChangedMaxPointBoundary;
    const auto closedWeight =
        closedMaxRef.mChangedMaxPointBoundary / combinedMax;
    const auto openWeight = openMaxRef.mChangedMaxPointBoundary / combinedMax;

    // Assert that the weights are sensible and add to 1.
    assert(closedWeight >= 0. && closedWeight <= 1.);
    assert(openWeight >= 0. && openWeight <= 1.);
    assert(qFuzzyCompare(closedWeight + openWeight, 1.));

    // Resize the QList, reset the grades
    mGrades = QList<Grade>(closedGradesRef.mGrades.size());

    // Transform takes a pair of values from the two input lists at the
    // same index and writes the result of getGradeAverage into the same
    // position in the new list.
    using namespace std::placeholders;
    std::ranges::transform(closedGradesRef.mGrades, openGradesRef.mGrades,
                           std::ranges::begin(mGrades),
                           std::bind(getGradeAverage, _1, closedWeight, _2,
                                     openWeight,
                                     configRef.mRequirePassAllParts));
}
