#pragma once

#include "node.hpp"

#include "data/csv_dump.hpp"

#include <QList>

#include <cstddef>

struct StudentData : Node<> {
    using Node<>::Node;

    /**
     * \brief Interprets the CsvDump as a RWTHOnline-csv-file and copies all
     * necessary info. Also runs sanity checks on the given data and throws on
     * error.
     *
     * \param c The constructor might remove unnecessary quotation marks from
     * some cells in this csv-file.
     *
     * \exception std::invalid_argument A sanity check on the given csv file
     * failed, see the description in the exception for details, or a std::stoi
     * conversion failed.
     * \exception std::out_of_range Thrown by std::stoi.
     */
    explicit StudentData(CsvDump &c) noexcept(false);

    QList<int> mRegistrationNr;
    QList<int> mAttemptNr;
    // todo: maybe also grade column

private:
    // internal helper functions
    void reserveArrays(qsizetype nrStudents) noexcept;
    void insertData(CsvDump &c, std::size_t regIdx,
                    std::size_t attemptIdx) noexcept(false);
};
