/**
 * \file csv_parser.hpp
 * \brief Declares functions and structs to read and store csv files from disk
 * in memory.
 */

#pragma once

#include "data/csv_dump.hpp"

#include <QSharedPointer>

#include <filesystem>

/**
 * \brief Reads the file at the given path into a \ref CsvDump if possible.
 *
 * Tries to open the file at the given path and throws an
 * `std::ios_base::failure` if that is not possible. Afterwards it checks if the
 * number of delimiters per line is consistent and throws an
 * `std::runtime_error` if that is not the case. Finally it stores the data in a
 * \ref CsvDump.
 *
 * \param[in] p     Path to the file.
 * \param[in] delim Delimiter character.
 *
 * \exception std::ios_base::failure Bad bit set while reading the file
 * \exception std::invalid_argument  The file could not be opened
 * \exception std::runtime_error     The number of delimiters per line is
 * inconsistent.
 *
 * \return Returns a pointer to the extracted CSV-data at the given path.
 */
QSharedPointer<CsvDump> readCsv(const std::filesystem::path &p,
                                char delim = ';');
