#include "input_model.hpp"

#include "file_io/csv_parser.hpp"

InputModel::InputModel(const FileType t) : mType(t) {}

bool InputModel::loadCsv(const std::filesystem::path &p) {

    // We could move this to a separate thread to keep the interface
    // responsive, but it only takes 1ms on my machine, so let's not
    // bother.
    try {
        const auto dump = readCsv(p);

        switch (mType) {
        default:
            assert(false);
            [[fallthrough]];
        case FileType::dynexite:

            mLoadedData.emplace<std::unique_ptr<DynexiteExam>>(
                new DynexiteExam(*dump));
            break;
        case FileType::rwth:
            mLoadedData.emplace<std::unique_ptr<StudentData>>(
                new StudentData(*dump));
            break;
        }

        return true;
    } catch (...) {
        // csv file read or object instantiation failed!
        // todo: look at error more closely, display what went wrong
        return false;
    }
}

std::variant<DynexiteExam *, StudentData *> InputModel::read() {
    std::variant<DynexiteExam *, StudentData *> ret;
    if (std::holds_alternative<std::unique_ptr<DynexiteExam>>(mLoadedData)) {

        ret = std::get<std::unique_ptr<DynexiteExam>>(mLoadedData).get();
    } else {
        ret = std::get<std::unique_ptr<StudentData>>(mLoadedData).get();
    }
    return ret;
}
