#pragma once

#include "data/nodes/dynexite_exam.hpp"
#include "data/nodes/student_data.hpp"
#include "util/enums.hpp"

#include <filesystem>
#include <memory>
#include <utility>
#include <variant>

class InputModel {
public:
    /**
     * \brief Construct a model that controls file loading for the given file
     * type.
     */
    explicit InputModel(FileType t);

    /**
     * \brief Validates that the user selected a correct file and stores the
     * content in the appropriate data structure in `mLoadedData`.
     *
     * \param p Path to the file
     *
     * \return Success
     */
    bool loadCsv(const std::filesystem::path &p);

    /**
     * \brief Which file type this model stores.
     */
    const FileType mType;

    /**
     * \brief Moves ownership of the stored data to the caller, is empty
     * afterwards.
     * \return The stored data.
     */
    [[nodiscard]] auto get() { return std::move(mLoadedData); }
    [[nodiscard]] std::variant<DynexiteExam *, StudentData *> read();

private:
    std::variant<std::unique_ptr<DynexiteExam>, std::unique_ptr<StudentData>>
        mLoadedData{};
};
