#pragma once

#include <compare>
#include <concepts>

struct Points {
    double p = 0.;

    [[nodiscard]] auto operator<=>(const Points &) const noexcept = default;
};

[[nodiscard]] constexpr Points operator+(const Points &a,
                                         const Points &b) noexcept {
    return Points{a.p + b.p};
}
[[nodiscard]] constexpr Points operator-(const Points &a,
                                         const Points &b) noexcept {
    return Points{a.p - b.p};
}

template <std::integral Int>
[[nodiscard]] constexpr Points operator/(const Points &a,
                                         const Int b) noexcept {
    return Points{a.p / static_cast<double>(b)};
}

[[nodiscard]] constexpr double operator/(const Points &a,
                                         const Points &b) noexcept {
    return a.p / b.p;
}

template <std::floating_point Float>
[[nodiscard]] constexpr Points operator*(const Points &a,
                                         const Float b) noexcept {
    return Points{a.p * static_cast<double>(b)};
}
