#pragma once

#include "model/input_model.hpp"
#include "util/enums.hpp"

#include <QGroupBox>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>

class LoadFileView : public QGroupBox {

    Q_OBJECT
public:
    explicit LoadFileView(FileType type, QWidget *parent = nullptr);

    /**
     * \brief Resets all user input.
     */
    void reset();

    void tempSetDeveloperPaths();

    [[nodiscard]] auto get() {
        reset();
        return mInputModel.get();
    }
    [[nodiscard]] auto read() { return mInputModel.read(); }

    /** \brief State of each file selection. */
    FileSelectState mFileState = FileSelectState::empty;

signals:
    /**
     * \brief Signals that the underlying data was changed.
     */
    void modelChange();

private:
    /** \brief Label that displays the path of a file selection. */
    QLabel mFileNameLabel{"Select File...", this};

    /** \brief Icon that displays the status of a file selection. */
    QLabel mFileParseStatusIcon{"", this};

    /** \brief Layout for all elements in this box. */
    QHBoxLayout mHLayout{this};

    /** \brief Button UI elements which open a dialog to select a file. */
    QPushButton mBrowseButton{"Browse...", this};

    /**
     * \brief Updates the icon and state of the given FileType to the new
     * status.
     *
     * \todo Use better icons.
     */
    void setFileStatus(FileSelectState status);

    InputModel mInputModel;

private slots:
    void handleBrowseButton();
};
