/**
 * \file main_window.hpp
 * \brief Declaration of the \ref MainWindow class.
 */

#pragma once

#include "grade_tab.hpp"
#include "view_settings.hpp"

#include "util/testing.hpp"
#include "view/boundary_window/boundary_window.hpp"
#include "view/input_window/input_window.hpp"

#include <QMainWindow>

struct DynexiteExam;
struct StudentData;

class QAction;
class QMenu;

/**
 * \brief Main display window.
 *
 * The MainWindow displays the exam results.
 */
class MainWindow : public QMainWindow {
    Q_OBJECT

public:
    /**
     * \brief Constructs a MainWindow object.
     *
     * Does not immediately draw itself, but instead creates and displays
     * the \ref InputWindow to get input files.
     *
     * \param[in] parent Parent widget.
     */
    explicit MainWindow(QWidget *parent = nullptr);

public slots:
    /**
     * \brief Loads the given exam and, if no errors occurred, displays it.
     */
    void resetToInputFiles(const DynexiteExam &exam,
                           const StudentData &students,
                           GradeBoundaryStepSize gradeBoundaryStepSize);

private slots:
    void menuNewFile();
    void menuOpenBoundaries();
    void menuOpenViewSettings();

private:
    /**
     * \brief Close all windows when main window is closed.
     */
    void closeEvent(QCloseEvent *event) override;

    void resizeEvent(QResizeEvent *event) override;

private:
    /**
     * Automatically input .bgk file if given
     */
    // void handleCommandLineArgs() const;

    /**
     * \brief The window where the user selects input files.
     */
    InputWindow mInputWindow;

    /**
     * \brief The window where the user selects input files.
     */
    BoundaryWindow mBoundaryWindow;

    /**
     * \brief Window with visual settings.
     */
    ViewSettingsWindow mViewSettings;

    /**
     * \brief Main tab that displays general exam results and lets the user
     * change passing limit and maximal points.
     *
     * \todo Since this is now the only tab, rename / move everything in here
     */
    GradeTab mGradeTab;

    /**
     * \brief Menu objects
     */
    QMenu *mFileMenu;
    QMenu *mViewMenu;
    QMenu *mExportMenu;
    QMenu *mExportGradingMenu;

    FRIEND_TEST(GuiDagTest, IntegrationTest);
    friend class GuiDagTest;
};
