#include "stat_window.hpp"

#include "data/data_manager.hpp"

#include <QLocale>

inline constexpr int vSpacerRow = 0;
inline constexpr int avgRow = 1;
inline constexpr int avgPassedRow = 2;

inline constexpr int textColumn = 0;
inline constexpr int middleSpacerColumn = 1;
inline constexpr int avgNumberColumn = 2;
inline constexpr int rightSpacerColumn = 2;

inline constexpr int nrRows = 3;
inline constexpr int nrColumns = 4;

StatWindow::StatWindow(QWidget *parent) : QWidget(parent), mLayout(this) {

    // These spacer items make the average grade labels appear in the bottom
    // left corner
    mLayout.addItem(
        new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding),
        vSpacerRow, 0, 1, nrColumns);
    mLayout.addItem(
        new QSpacerItem(10, 0, QSizePolicy::Maximum, QSizePolicy::Maximum), 1,
        middleSpacerColumn, nrRows, 1);
    mLayout.addItem(
        new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Expanding),
        1, rightSpacerColumn, nrRows, 1);

    {
        const auto avgLabel = new QLabel("# Average grade (Attended):");
        avgLabel->setTextFormat(Qt::MarkdownText);
        mLayout.addWidget(avgLabel, avgRow, textColumn);
    }
    {
        const auto avgPassedLabel = new QLabel("# Average grade (Passed):");
        avgPassedLabel->setTextFormat(Qt::MarkdownText);
        mLayout.addWidget(avgPassedLabel, avgPassedRow, textColumn);
    }
    {
        mAverageGrade.setTextFormat(Qt::MarkdownText);
        mLayout.addWidget(&mAverageGrade, avgRow, avgNumberColumn);
    }

    {
        mAverageGradePassed.setTextFormat(Qt::MarkdownText);
        mLayout.addWidget(&mAverageGradePassed, avgPassedRow, avgNumberColumn);
    }

    // receive update when grades change
    connect(&DataManager::getInstance(), &DataManager::gradesChanged, this,
            &StatWindow::gradesChanged);
}

void StatWindow::gradesChanged(const EnumStorage<bool> &) {
    const auto cur = DataManager::getCurrent();
    if (!cur) {
        mAverageGrade.setText("# -");
        mAverageGradePassed.setText("# -");
        return;
    }

    // Number of decimal places for average grade is always 2, independent of
    // the grade boundary step size (todo: make configurable)
    mAverageGrade.setText(
        "# " + QLocale().toString(cur->mGradeAverage.mAverageGrade, 'f', 2));
    mAverageGradePassed.setText(
        "# " +
        QLocale().toString(cur->mGradeAverage.mAverageGradePassed, 'f', 2));
}
